@@ -10,9 +10,9 @@
 #include <regex.h>
 
 #include "mt.h"
+#include "utils.h"
 #include "version.h"
 
-
 void error_exit(char *format, ...)
 {
 	va_list ap;
@@ -26,8 +26,10 @@ void error_exit(char *format, ...)
 	(void)vfprintf(stderr, format, ap);
 	va_end(ap);
 	if (errno) fprintf(stderr, "errno: %d=%s (if applicable)\n", errno, strerror(errno));
-	fprintf(stderr, "\nPress enter to exit.\n");
-	getchar();
+
+	dump_mem(SIGHUP);
+
+	fflush(NULL);
 
 	(void)kill(0, SIGTERM); /* terminate every process in the process group of the current process */
 
