@@ -153,11 +153,11 @@ int start_proc(proginfo *cur, int initial_tail)
 			signal(SIGTERM, SIG_DFL);
 
 			/* sleep if requested and only when 2nd or 3d (etc.) execution time */
-			if (cur -> restart && cur -> first == 0)
-				sleep(cur -> restart);
+			if (cur -> restart.restart && cur -> restart.first == 0)
+				sleep(cur -> restart.restart);
 
 			/* connect slave-fd to stdin/out/err */
-			setup_for_childproc(fd_slave, 1, term_t_to_string(cur -> term_emul));
+			setup_for_childproc(fd_slave, 1, term_t_to_string(cur -> cdef.term_emul));
 
 			/* start process */
 			if (-1 == execlp(shell, shell, "-c", cur -> filename, (void *)NULL)) error_exit("start_proc: execlp of %s failed\n", cur -> filename);
@@ -180,7 +180,7 @@ int start_proc(proginfo *cur, int initial_tail)
 		cur -> wfd = fd_master;
 
 		/* next time, sleep */
-		cur -> first = 0;
+		cur -> restart.first = 0;
 	}
 	else
 	{
@@ -324,3 +324,87 @@ void init_children_reaper(void)
 	/* init list of pids to watch for exit */
 	memset(children_list, 0x00, sizeof(children_list));
 }
+
+pid_t exec_with_pipe(char *command, int pipe_to_proc[], int pipe_from_proc[])
+{
+	pid_t pid = -1;
+
+	if (pipe(pipe_to_proc) == -1)
+		error_exit("exec_with_pipe: error creating pipe 1\n");
+
+	if (pipe(pipe_from_proc) == -1)
+		error_exit("exec_with_pipe: error creating pipe 2\n");
+
+	if ((pid = fork()) == -1)
+		error_exit("exec_with_pipemain: fork() failed\n");
+
+	if (pid == 0)
+	{
+		close(0);
+		if (mydup(pipe_to_proc[0]) == -1) error_exit("exec_with_pipe: dup() failed\n");
+		close(pipe_to_proc[1]); /* will not write to itself, only parent writes to it */
+		close(1);
+		close(2);
+		if (mydup(pipe_from_proc[1]) == -1) error_exit("exec_with_pipe: dup() failed\n");
+		if (mydup(pipe_from_proc[1]) == -1) error_exit("exec_with_pipe: dup() failed\n");
+		close(pipe_from_proc[0]);
+
+		/* start process */
+/*		if (-1 == execlp(shell, shell, "-c", command, (void *)NULL)) error_exit("main: execlp of %s failed\n", command); */
+		if (-1 == execlp(command, command, (void *)NULL)) error_exit("bla\n");
+
+		/* if execlp returns, an error occured */
+		error_exit("exec_with_pipe: error while starting process '%s'!\n", command);
+	}
+
+	return pid;
+}
+
+pid_t exec_with_pty(char *command, int *fd)
+{
+	int fd_master = -1, fd_slave = -1;
+	pid_t pid = get_pty_and_fork(&fd_master, &fd_slave);
+	if (-1 == pid) error_exit("start_proc: failed to create pseudo-tty & fork\n");
+
+	if (pid == 0)
+	{
+		setpgid(0,0);
+
+		close(fd_master);
+
+		/* connect slave-fd to stdin/out/err */
+		setup_for_childproc(fd_slave, 1, "dumb");
+
+		/* start process */
+		if (-1 == execlp(command, command, (void *)NULL)) error_exit("exec_with_pty: execlp of %s failed\n", command);
+	}
+
+	*fd = fd_master;
+
+#if defined(sun) || defined(__sun) || defined(AIX) || defined(_HPUX_SOURCE) || defined(OSF1) || defined(scoos)
+	/* see start_proc */
+#else
+	if (myclose(fd_slave) == -1) error_exit("exec_with_pty: error closing slave-fd (pseudo tty, fd %d)\n", fd_slave);
+#endif
+
+	return pid;
+}
+
+void exec_script(script *pscript)
+{
+	if (pscript -> pid == 0)
+	{
+		int to[2], from[2];
+
+		pscript -> pid = exec_with_pipe(pscript -> script, to, from);
+		pscript -> fd_r = from[0];
+		pscript -> fd_w = to[1];
+/*
+		int fd;
+
+		pscript -> pid = exec_with_pty(pscript -> script, &fd);
+		pscript -> fd_r =
+		pscript -> fd_w = fd;
+	*/
+	}
+}
