@@ -15,23 +15,29 @@ char *nsubwindows = NULL;
 char *set_title = NULL;
 char *shell = "/bin/sh";
 char *tail = "tail";
-char *ts_format = "%b %2d %H:%M:%S";
-char *statusline_ts_format = "%b %2d %H:%M:%S %Y";
-char *cnv_ts_format = "%b %2d %H:%M:%S %Y";
+char *ts_format = "%b %d %H:%M:%S";
+char *statusline_ts_format = "%b %d %H:%M:%S %Y";
+char *cnv_ts_format = "%b %d %H:%M:%S %Y";
+char *line_ts_format = "%b %d %H:%M:%S %Y ";
 char **color_names = NULL;
 char *replace_by_markerline = NULL;
 const char *F1 = "For help at any time press F1.";
 int *n_win_per_col = NULL;
 int *vertical_split = NULL;
 color_scheme *cschemes = NULL;
-const char *version_str = " --*- multitail " VERSION " (C) 2003-2006 by folkert@vanheusden.com -*--";
+const char *version_str = " --*- multitail " VERSION " (C) 2003-2007 by folkert@vanheusden.com -*--";
 conversion *conversions = NULL;
 keybinding *keybindings = NULL;
 pars_per_file *ppf = NULL;
 proginfo *pi = NULL;
 check_dir_glob *cdg = NULL;
 int n_cdg = 0;
+editscheme *pes = NULL;
+int n_es = 0;
+char *defaultcscheme = NULL;
+proginfo *terminal_index = NULL;
 
+int terminal_main_index = -1;
 int default_color_scheme = -1;
 int max_y, max_x;
 int min_n_bufferlines = 25;
@@ -39,12 +45,10 @@ int mode_statusline = 1;
 int n_children = 0;
 int n_conversions = 0;
 int n_cschemes = 0;
-int n_cschemes;
 int n_keybindings = 0;
 int n_pars_per_file = 0;
 int n_splitlines = 0;
 int nfd = 0;
-int terminal_index = -1;
 int update_interval = 0;
 int n_colors_defined = 0;
 int check_for_mail = 5;	/* check for mail every 5 seconds */
@@ -52,6 +56,13 @@ int default_maxnlines = 0;
 int default_maxbytes = 0;
 int popup_refresh_interval = 5; /* every 5 seconds */
 int default_line_wrap_offset = 0;
+int inverse_attrs = A_REVERSE;
+int beep_interval = -1;
+int linecounter_for_beep = 0;
+int did_n_beeps = 0;
+int default_min_shrink = 10;
+int wordwrapmaxlength = 10;
+int default_bg_color = -1;
 
 double heartbeat_interval = 0.0;
 double heartbeat_t = 0.0;
@@ -69,8 +80,20 @@ myattr_t changeline_attrs = { -1, -1 };
 myattr_t idleline_attrs   = { -1, -1 };
 myattr_t msgline_attrs    = { -1, -1 };
 myattr_t statusline_attrs = { -1, -1 };
+myattr_t splitline_attrs  = { -1, -1 };
 colorpairs cp;
 mode_t def_umask = 0022;
+history_t search_h = { NULL, 0, NULL };
+history_t cmdfile_h = { NULL, 0, NULL };
+mybool_t re_case_insensitive = MY_FALSE;
+chtype box_bottom_left_hand_corner=0;
+chtype box_bottom_right_hand_corner=0;
+chtype box_bottom_side=0;
+chtype box_left_side=0;
+chtype box_right_side=0;
+chtype box_top_left_hand_corner=0;
+chtype box_top_right_hand_corner=0;
+chtype box_top_side=0;
 
 beeb_t beep_method = BEEP_BEEP;
 double beep_popup_length = 0.0;
@@ -100,12 +123,16 @@ char changeline_char = '-';
 char idleline_char   = '-';
 char msgline_char   = '-';
 char default_bufferwhat = 'f';
-char re_case_insensitive = 0;
 char load_global_config = 1;
 char default_linewrap = 'a';
 char default_follow_filename = 0;
 char do_not_close_closed_windows = 0;
 char suppress_empty_lines = 1;
+char splitline_mode = SL_REGULAR;
+char abort_key = 7;
+char exit_key = 3;
+char default_sb_showwinnr = 0;
+char reuse_searchstring = 1;
 
 regex_t global_highlight_re;
 char *global_highlight_str = NULL;
