@@ -23,7 +23,8 @@ extern char tab_width;
 extern time_t mt_started;
 extern int *vertical_split;
 extern int *n_win_per_col;
-extern int terminal_index;
+extern proginfo *terminal_index;
+extern int terminal_main_index;
 extern char prev_term_char;
 extern int n_keybindings;
 extern keybinding *keybindings;
@@ -89,7 +90,7 @@ extern int n_children;
 extern term_t term_type;
 extern regex_t global_highlight_re;
 extern char *global_highlight_str;
-extern char re_case_insensitive;
+extern mybool_t re_case_insensitive;
 extern filterscheme *pfs;
 extern int n_fs;
 extern char load_global_config;
@@ -98,3 +99,30 @@ extern int default_line_wrap_offset;
 extern char default_follow_filename;
 extern char do_not_close_closed_windows;
 extern char suppress_empty_lines;
+extern editscheme *pes;
+extern int n_es;
+extern char splitline_mode;
+extern myattr_t splitline_attrs;
+extern int inverse_attrs;
+extern char abort_key;
+extern char exit_key;
+extern int beep_interval;
+extern int linecounter_for_beep;
+extern int did_n_beeps;
+extern char *defaultcscheme;
+extern char *line_ts_format;
+extern int default_min_shrink;
+extern char default_sb_showwinnr;
+extern int wordwrapmaxlength;
+extern history_t search_h;
+extern history_t cmdfile_h;
+extern int default_bg_color;
+extern char reuse_searchstring;
+extern chtype box_bottom_left_hand_corner;
+extern chtype box_bottom_right_hand_corner;
+extern chtype box_bottom_side;
+extern chtype box_left_side;
+extern chtype box_right_side;
+extern chtype box_top_left_hand_corner;
+extern chtype box_top_right_hand_corner;
+extern chtype box_top_side;
