@@ -5,6 +5,14 @@
 #define MAX_N_COLUMNS		15	/* max number of columns */
 #define DEFAULT_TAB_WIDTH	4
 #define DEFAULT_COLORPAIRS	8
+#define MAX_BACKTRACE_LENGTH	256
+#define SCRIPT_IO_BUFFER_SIZE	4096
+#define CONFIG_READ_BUFFER	4096
+#define HISTORY_IO_BUFFER	4096
+
+#define SL_REGULAR	1
+#define SL_NONE		0
+#define SL_ATTR		2
 
 #define LOADAVG_STR_LEN		20
 #define AMOUNT_STR_LEN		(3 + 2 + 1)
@@ -43,13 +51,17 @@ typedef enum { TERM_IGNORE = 0, TERM_XTERM, TERM_ANSI /* or vt100 */} term_t;
 /* it seems the default HP-UX c-compiler doesn't understand atoll and
  * strtoll while it does understand 'long long'
  */
-#if defined(_HPUX_SOURCE)
-#ifndef atoll
-#define atoll(x)	atol(x)
+#if defined(_HPUX_SOURCE) || defined(__APPLE__)
+	#ifndef atoll
+		#define atoll(x)	atol(x)
+	#endif
 #endif
+
 #ifndef strtoll
-#define strtoll(x, y, z)	strtol(x, y, z)
+	#define strtoll(x, y, z)	strtol(x, y, z)
 #endif
+#ifndef atoll
+      #define atoll(a) strtoll((a), (char **)NULL, 10)
 #endif
 
 /* Tru64 workaround */
@@ -70,10 +82,25 @@ typedef enum { TERM_IGNORE = 0, TERM_XTERM, TERM_ANSI /* or vt100 */} term_t;
 #define MARKER_MSG	(proginfo *)-3
 #define IS_MARKERLINE(x)	((x) == MARKER_REGULAR || (x) == MARKER_CHANGE || (x) == MARKER_IDLE || (x) == MARKER_MSG)
 
-typedef enum { SEL_WIN = 1, SEL_SUBWIN, SEL_FILES, SEL_CSCHEME } selbox_type_t;
+typedef enum { SEL_WIN = 1, SEL_SUBWIN, SEL_FILES, SEL_CSCHEME, SEL_HISTORY } selbox_type_t;
 
 typedef enum { TT_ATIME = 1, TT_MTIME, TT_CTIME } time_field_t;
 
+typedef struct
+{
+	char *history_file;
+	int history_size;
+
+	char **history;
+} history_t;
+
+typedef struct
+{
+	int *elements;
+	int n;
+	int size;
+} int_array_t;
+
 typedef struct
 {
 	char *glob_str;
@@ -133,6 +160,15 @@ typedef struct
 	int match_count;
 } strip_t;
 
+typedef struct
+{
+	char *es_name;
+	char *es_desc;
+
+	int n_strips;
+	strip_t *strips;
+} editscheme;
+
 #define MAX_COLORS_PER_LINE 80
 
 typedef struct
@@ -159,6 +195,57 @@ typedef struct {
 	pid_t pid;
 } redirect_t;
 
+typedef struct
+{	
+	time_t lastevent;
+	double prev_deltat, total_deltat;
+	double med;
+	double dev;
+	char sccfirst;
+	double scc, sccu0, scclast, scct1, scct2, scct3;
+	int n_events;
+
+	time_t start_ts;
+	long long int bytes_processed;
+} statistics_t;
+
+typedef struct
+{
+	char colorize;
+	char field_nr;
+	char *field_del;
+	int_array_t color_schemes;
+	myattr_t attributes;
+	char alt_col;
+	myattr_t alt_col_cdev1, alt_col_cdev2;
+	char syslog_noreverse;
+
+	term_t term_emul;
+} cdef_t;
+
+typedef struct
+{
+	char **bcur, **bprev;
+	int ncur, nprev;
+} diff_t;
+
+typedef struct
+{
+	int restart;
+	char restart_clear; /* clear after each iteration? */
+	char is_restarted;
+	char first;
+	char do_diff;
+	diff_t diff;
+} restart_t;
+
+typedef struct
+{
+	char suppress_repeating_lines;
+	char *last_line;
+	int n_times_repeated;
+} repeatinglines_t;
+
 typedef struct _subwindow_
 {
 	char *filename;
@@ -176,14 +263,13 @@ typedef struct _subwindow_
 
 	off64_t last_size;
 
-	term_t term_emul;
-
 	char cont;		/* "re-connect" lines with \ at the end */
 
 	char add_timestamp;
 
-	int *conversions;
-	int n_conversions;
+	char *label;		/* put in front of each line */
+
+	int_array_t conversions;
 
 	char paused;
 	char closed;
@@ -198,47 +284,30 @@ typedef struct _subwindow_
 	int win_height;
 
 	/* repeatingly start a program */
-	int restart;
-	char restart_clear; /* clear after each iteration? */
-	char is_restarted;
-	char first;
-	char do_diff;
-	char **bcur, **bprev;
-	int ncur, nprev;
+	restart_t restart;
 
 	int initial_n_lines_tail;
 
 	int mark_interval;
 
-	char suppress_repeating_lines;
-	char *last_line;
-	int n_times_repeated;
+	repeatinglines_t repeat;
 
-	char colorize;
-	char field_nr;
-	char *field_del;
-	int *color_schemes;
-	int n_color_schemes;
-	myattr_t attributes;
-	char alt_col;
-	myattr_t alt_col_cdev1, alt_col_cdev2;
-	char syslog_noreverse;
+	cdef_t cdef;
 
 	char hidden;
 	char follow_filename;
 	char retry_open;
 
 	int close_idle;
-	time_t lastevent;
-	double prev_deltat, total_deltat;
-	double med;
-	double dev;
-	char sccfirst;
-	double scc, sccu0, scclast, scct1, scct2, scct3;
-	int n_events;
 
-	time_t start_ts;
-	long long int bytes_processed;
+	statistics_t statistics;
+
+	struct
+	{
+		int beep_interval;
+		int linecounter_for_beep;
+		int did_n_beeps;
+	} beep;
 
 	NEWWIN *status;
 	NEWWIN *data;
@@ -278,29 +347,64 @@ typedef enum {
 	CSREFLAG_CMP_VAL_EQUAL		/* compare with value: value equal to what is configured? */
 } csreflag_t;
 
+typedef struct
+{
+	char *script;
+	pid_t pid;
+	int fd_r, fd_w;
+} script;
+
+typedef struct
+{
+	myattr_t attrs1;
+	myattr_t attrs2;
+	mybool_t use_alternating_colors;
+	int ac_index;
+} cse_main;
+
+typedef struct
+{
+	cse_main cdef;
+
+	regex_t regex;
+	csreflag_t flags;
+	double cmp_value;
+
+	mybool_t merge_color;
+
+} color_scheme_entry;
+
 typedef struct
 {
 	char *name;
 	char *descr;
 
+	script color_script;
+
 	int n;
-	myattr_t *attrs;
-        regex_t *regex;
-	csreflag_t *flags;
-	double *cmp_value;
+	color_scheme_entry *pentries;
 } color_scheme;
 
-typedef enum { CONVTYPE_IP4TOHOST = 1, CONVTYPE_EPOCHTODATE, CONVTYPE_ERRNO, CONVTYPE_HEXTODEC, CONVTYPE_DECTOHEX, CONVTYPE_TAI64NTODATE } conversion_t;
+typedef enum { CONVTYPE_IP4TOHOST = 1, CONVTYPE_EPOCHTODATE, CONVTYPE_ERRNO, CONVTYPE_HEXTODEC, CONVTYPE_DECTOHEX, CONVTYPE_TAI64NTODATE, CONVTYPE_SCRIPT, CONVTYPE_ABBRTOK } conversion_t;
+typedef struct
+{
+	conversion_t type;
+	regex_t regex;
+	int match_count;
+} conversion_bundle_t;
 typedef struct
 {
 	char *name;
 
 	int n;
-	conversion_t *type;
-	regex_t *regex;
-	int *match_count;
+	conversion_bundle_t *pcb;
+
+	/* conversion script */
+	script *pcs;
 } conversion;
 
+typedef enum { SCHEME_CONVERSION = 1, SCHEME_COLOR } scheme_t;
+
 typedef struct cv_off
 {
 	int start, end;
@@ -310,21 +414,31 @@ typedef struct cv_off
 typedef struct
 {
 	int n_colorschemes;
-	int *colorschemes;
+	char **colorschemes;
+
 	int buffer_maxnlines;
 	int buffer_bytes;
+
 	char change_win_marker;
+
 	regex_t regex;
 	char *re_str;
-	int n_filterschemes;
-	int *filterschemes;
+
+	int_array_t filterschemes;
+
+	int_array_t editschemes;
+
+	int n_conversion_schemes;
+	char **conversion_schemes;
 } pars_per_file;
 
 typedef struct
 {
 	regoff_t start;
 	regoff_t end;
+
 	myattr_t attrs;
+	mybool_t merge_color;
 } color_offset_in_line;
 
 typedef struct
@@ -336,7 +450,7 @@ typedef struct
 void do_exit(void);
 char * select_file(char *input, int what_help);
 char check_no_suppress_lines_filter(proginfo *cur);
-void color_print(NEWWIN *win, proginfo *cur, char *string, regmatch_t *matches, int matching_regex, mybool_t force_to_winwidth, int start_at_offset, int end_at_offset);
+void color_print(int f_index, NEWWIN *win, proginfo *cur, char *string, regmatch_t *matches, int matching_regex, mybool_t force_to_winwidth, int start_at_offset, int end_at_offset, double ts, char show_window_nr);
 int select_window(int what_help, char *heading);
 char check_filter(proginfo *cur, char *string, regmatch_t **pmatch, char **error, int *matching_regex, char do_re, char *display);
 int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char shift_cursor);
@@ -346,8 +460,8 @@ void redraw_statuslines(void);
 void buffer_replace_pi_pointers(int f_index, proginfo *org, proginfo *new);
 char delete_entry(int f_index, proginfo *sub);
 char * key_to_keybinding(char what);
-void do_buffer(int f_index, proginfo *cur, char *string, char filter_match);
-void do_print(int f_index, proginfo *cur, char *string, regmatch_t *matches, int matching_regex);
+void do_buffer(int f_index, proginfo *cur, char *string, char filter_match, double now);
+void do_print(int f_index, proginfo *cur, char *string, regmatch_t *matches, int matching_regex, double ts);
 void do_set_bufferstart(int f_index, char store_what_lines, int maxnlines);
 void emit_myattr_t(FILE *fh, myattr_t what);
 void update_statusline(NEWWIN *status, int win_nr, proginfo *cur);
@@ -356,7 +470,7 @@ void check_proc_sigh(int sig);
 void info(void);
 void statistics(void);
 int emit_to_buffer_and_term(int f_index, proginfo *cur, char *line);
-void add_pars_per_file(char *re, int cs, int n_buffer_lines, int buffer_bytes, char change_win_marker, int fs);
+void add_pars_per_file(char *re, char *colorscheme, int n_buffer_lines, int buffer_bytes, char change_win_marker, int fs, int es, char *conversion_scheme);
 void version(void);
 void usage(void);
 void create_new_win(proginfo **cur, int *nr);
