@@ -26,6 +26,8 @@
 #include "utils.h"
 #include "help.h"
 #include "globals.h"
+#include "history.h"
+#include "ui.h"
 
 
 void wrong_key(void)
@@ -61,6 +63,8 @@ int ask_yes_no(int what_help, NEWWIN *popup)
 	{
 		int c = toupper(wait_for_keypress(what_help, 0, popup, 0));
 
+		if (c == abort_key) return -1;
+
 		switch(c)
 		{
 		case 'Y':
@@ -70,7 +74,6 @@ int ask_yes_no(int what_help, NEWWIN *popup)
 		case 'N':
 			return 0;
 
-		case 7:
 		case 'Q':
 			return -1;
 		}
@@ -107,12 +110,12 @@ void myattr_off(NEWWIN *win, myattr_t attrs)
 		wattroff(win -> win, attrs.attrs);
 }
 
-void inverse_on(NEWWIN *win)
+void ui_inverse_on(NEWWIN *win)
 {
 	wattron(win -> win, A_REVERSE);
 }
 
-void inverse_off(NEWWIN *win)
+void ui_inverse_off(NEWWIN *win)
 {
 	wattroff(win -> win, A_REVERSE);
 }
@@ -131,12 +134,15 @@ void draw_line(NEWWIN *win, linepos_t where)
 		mvwhline(win -> win, my-1, 0, ' ', mx);
 }
 
-char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_width, char numbers_only, char *input_string, int what_help, char first_char)
+char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_width, char numbers_only, char *input_string, int what_help, char first_char, history_t *ph, mybool_t *pcase_insensitive)
 {
 	char *string = (char *)mymalloc(max_width + 1, "edit_string: edit buffer");
 	int str_pos = 0, x = 0;
 	int line_width = win_width;
 
+	if (pcase_insensitive)
+		mvwprintw(win -> win, win_y + 1, win_x - 1, "[%c] case insensitive (press TAB)", *pcase_insensitive?'X':' ');
+
 	if (input_string)
 	{
 		int input_string_len = strlen(input_string), copy_len = min(min(win_width, max_width), input_string_len);
@@ -163,7 +169,7 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 		int prev_str_pos = str_pos;
 		int c;
 
-		if (first_char != -1)
+		if (first_char != (char)-1)
 		{
 			c = first_char;
 			first_char = -1;
@@ -178,7 +184,7 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 			break;
 
 		/* abort */
-		if (c == 7 || c == 17 || c == 24) /* ^g / ^q / ^x */
+		if (c == abort_key || c == 17 || c == 24) /* ^g / ^q / ^x */
 		{
 			string[0] = 0x00;
 			break;
@@ -205,8 +211,13 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 				}
 			}
 			break;
-		case 9:			/* tab (filename completion) */
-			if (numbers_only)
+		case 9:			/* tab (filename completion/switch to case insensitve field) */
+			if (pcase_insensitive)
+			{
+				ask_case_insensitive(pcase_insensitive);
+				force_redraw = 1;
+			}
+			else if (numbers_only)
 			{
 				wrong_key();
 			}
@@ -220,7 +231,6 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 					strncpy(string, file, max_width);
 					string[max_width] = 0x00;
 					myfree(file);
-					str_pos = 0;
 				}
 
 				dummy = strlen(string);
@@ -243,7 +253,7 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 			str_pos = x = 0;
 			force_redraw = 1;
 			break;
-		case 23:		/* ^W FIXME: delete word */
+		case 23:		/* ^W delete word */
 			{
 				int spos = str_pos + x;
 				int dpos = spos;
@@ -279,6 +289,40 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 			}
 			break;
 
+		case KEY_DOWN:		/* cursor down */
+		case 18:		/* ^R */
+			if (ph == NULL || ph -> history_size <= 0 || ph -> history_file == NULL)
+			{
+				wrong_key();
+			}
+			else
+			{
+				int dummy;
+				char *hs = search_history(ph, string);
+
+				if (hs)
+				{
+					strncpy(string, hs, max_width);
+					string[max_width] = 0x00;
+					myfree(hs);
+				}
+
+				dummy = strlen(string);
+				if (dummy > line_width)
+				{
+					str_pos = dummy - (line_width / 2);
+					x = (line_width / 2);
+				}
+				else
+				{
+					str_pos = 0;
+					x = dummy;
+				}
+
+				force_redraw = 1;
+			}
+			break;
+
 		case KEY_BACKSPACE:
 			{
 				int spos = str_pos + x;
@@ -384,17 +428,23 @@ char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_wid
 				mvwprintw(win -> win, win_y, win_x + loop, " ");
 			mvwprintw(win -> win, win_y, win_x, dummy);
 			myfree(dummy);
+			if (pcase_insensitive)
+				mvwprintw(win -> win, win_y + 1, win_x, "%c", *pcase_insensitive?'X':' ');
 			force_redraw = 0;
 		}
 		wmove(win -> win, win_y, win_x + x);
 		mydoupdate();
 	}
 
-	if (strlen(string) == 0)
+	if (string[0] == 0x00)
 	{
 		myfree(string);
 		string = NULL;
 	}
+	else if (ph != NULL)
+	{
+		history_add(ph, string);
+	}
 
 	return string;
 }
@@ -409,7 +459,7 @@ NEWWIN * create_popup(int n_lines, int n_colls)
 	newwin = mynewwin(n_lines, n_colls, olines, ocols);
 
 	werase(newwin -> win);
-	box(newwin -> win, 0, 0);
+	wborder(newwin -> win, box_left_side, box_right_side, box_top_side, box_bottom_side, box_top_left_hand_corner, box_top_right_hand_corner, box_bottom_left_hand_corner, box_bottom_right_hand_corner);
 
 	show_panel(newwin -> pwin);
 
@@ -463,6 +513,9 @@ NEWWIN * mynewwin(int nlines, int ncols, int begin_y, int begin_x)
 	if (bright_colors)
 		wattron(nwin -> win, A_BOLD);
 
+	if (default_bg_color != -1)
+		wbkgdset(nwin -> win, COLOR_PAIR(find_or_init_colorpair(-1, default_bg_color, 1)));
+
 	return nwin;
 }
 
@@ -484,9 +537,9 @@ void escape_print(NEWWIN *win, int y, int x, char *str)
 			else
 			{
 				if (!inv)
-					inverse_on(win);
+					ui_inverse_on(win);
 				else
-					inverse_off(win);
+					ui_inverse_off(win);
 
 				inv = 1 - inv;
 			}
@@ -533,7 +586,7 @@ void escape_print(NEWWIN *win, int y, int x, char *str)
 		}
 	}
 
-	if (inv) inverse_off(win);
+	if (inv) ui_inverse_off(win);
 	if (ul)  wattroff(win -> win, A_UNDERLINE);
 	if (bold)  wattroff(win -> win, A_BOLD);
 }
@@ -562,8 +615,7 @@ int find_colorpair(int fgcolor, int bgcolor, char fuzzy)
 
 	for(loop=0; loop<cp.n_def; loop++)
 	{
-		if (cp.fg_color[loop] == fgcolor &&
-				cp.bg_color[loop] == bgcolor)
+		if (cp.fg_color[loop] == fgcolor && cp.bg_color[loop] == bgcolor)
 		{
 			return loop;
 		}
@@ -571,7 +623,7 @@ int find_colorpair(int fgcolor, int bgcolor, char fuzzy)
 
 	if (fuzzy)
 	{
-		for(loop=0; loop<cp.n_def; loop++)
+		for(loop=cp.n_def-1; loop>=0; loop++)
 		{
 			if (cp.fg_color[loop] == fgcolor && cp.bg_color[loop] == -1)
 				return loop;
@@ -637,7 +689,7 @@ int colorstr_to_nr(char *str)
 
 	for(loop=0; loop<n_colors_defined; loop++)
 	{
-		if (color_names[loop] && strcasecmp(color_names[loop], str) == 0)
+		if (color_names[loop] && strcmp(color_names[loop], str) == 0)
 			return loop;
 	}
 
@@ -660,8 +712,6 @@ int attrstr_to_nr(char *str)
 
 		if (strcmp(str, "bold") == 0)
 			attr |= A_BOLD;
-		else if (strcmp(str, "blink") == 0)
-			attr |= A_BLINK;
 		else if (strcmp(str, "reverse") == 0 || strcmp(str, "inverse") == 0)
 			attr |= A_REVERSE;
 		else if (strcmp(str, "underline") == 0)
@@ -670,13 +720,15 @@ int attrstr_to_nr(char *str)
 			attr |= A_NORMAL;
 		else if (strcmp(str, "dim") == 0)
 			attr |= A_DIM;
+		else if (strcmp(str, "blink") == 0)
+			attr |= A_BLINK;
 		else
 			error_exit("'%s' is not recognized as an attribute\n", str);
 
-		if (slash)
-			str = slash + 1;
-		else
+		if (!slash)
 			break;
+
+		str = slash + 1;
 	}
 
 	return attr;
@@ -894,6 +946,7 @@ char * emulate_terminal(char *string, color_offset_in_line **cmatches, int *n_cm
 					*cmatches = (color_offset_in_line *)myrealloc(*cmatches, cur_n_cmatches * sizeof(color_offset_in_line), "emulate_terminal: cmatches");
 				}
 
+				memset(&(*cmatches)[*n_cmatches], 0x00, sizeof(color_offset_in_line));
 				(*cmatches)[*n_cmatches].start = new_offset;
 				(*cmatches)[*n_cmatches].end = -1;
 				(*cmatches)[*n_cmatches].attrs = ci;
@@ -954,7 +1007,7 @@ void init_colornames(void)
 	dummy = min(256, COLORS);
 	if (use_colors)
 	{
-		color_names = (char **)mymalloc(dummy * sizeof(char *), "init_colors: color_names");
+		color_names = (char **)mymalloc(dummy * sizeof(char *), "init_colornames: color_names");
 		memset(color_names, 0x00, dummy * sizeof(char *));
 
 		color_names[COLOR_RED]    = "red";
