@@ -4,10 +4,10 @@ void color_on(NEWWIN *win, int colorpair_index);
 void color_off(NEWWIN *win, int colorpair_index);
 void myattr_on(NEWWIN *win, myattr_t attr);
 void myattr_off(NEWWIN *win, myattr_t attr);
-void inverse_on(NEWWIN *win);
-void inverse_off(NEWWIN *win);
+void ui_inverse_on(NEWWIN *win);
+void ui_inverse_off(NEWWIN *win);
 void draw_line(NEWWIN *win, linepos_t where);
-char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_width, char numbers_only, char *input_string, int what_help, char first_char);
+char * edit_string(NEWWIN *win, int win_y, int win_x, int win_width, int max_width, char numbers_only, char *input_string, int what_help, char first_char, history_t *ph, mybool_t *pcase_insensitive);
 NEWWIN * create_popup(int n_lines, int n_colls);
 void delete_popup(NEWWIN *mywin);
 void mydelwin(NEWWIN *win);
