@@ -21,6 +21,7 @@
 #include <sys/loadavg.h>
 #endif
 #include <dirent.h>
+#include <ctype.h>
 
 #include "version.h"
 #include "error.h"
@@ -57,7 +58,7 @@ int myrand(int max)
 	return (int)(((double)max * (double)rand()) / (double)RAND_MAX);
 }
 
-ssize_t WRITE(int fd, char *whereto, size_t len)
+ssize_t WRITE(int fd, char *whereto, size_t len, char *for_whom)
 {
 	ssize_t cnt=0;
 
@@ -70,7 +71,7 @@ ssize_t WRITE(int fd, char *whereto, size_t len)
 		if (rc == -1)
 		{
 			if (errno != EINTR && errno != EAGAIN)
-				error_exit("WRITE: Problem writing to filedescriptor\n");
+				error_exit("WRITE: Problem writing to filedescriptor while processing %s\n", for_whom);
 		}
 		else if (rc == 0)
 		{
@@ -261,9 +262,13 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 
 int file_exist(char *filename)
 {
-	struct stat64 buf;
+	struct stat buf;
+	int rc = stat(filename, &buf);
 
-	return stat64(filename, &buf);
+	if (rc == -1 && errno != ENOENT)
+		error_exit("file_exist(%s) failed\n", filename);
+
+	return rc;
 }
 
 char * convert_regexp_error(int error, const regex_t *preg)
@@ -498,14 +503,15 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 
 void setup_for_childproc(int fd, char close_fd_0, char *term)
 {
-	char *dummy = (char *)mymalloc(strlen(term) + 6, "setup_for_childproc: TERM env var");
+	int term_len = strlen(term) + 5;
+	char *dummy = (char *)mymalloc(term_len + 1, "setup_for_childproc: TERM env var");
 
 	if (close_fd_0) if (-1 == myclose(0)) error_exit("setup_for_childproc: close failed\n");
 	if (-1 == myclose(1)) error_exit("setup_for_childproc: close failed\n");
 	if (-1 == myclose(2)) error_exit("setup_for_childproc: close failed\n");
-	if (close_fd_0) if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
-	if (-1 == dup(fd)) error_exit("setup_for_childproc: dup failed\n");
+	if (close_fd_0) if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
+	if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
+	if (-1 == mydup(fd)) error_exit("setup_for_childproc: dup failed\n");
 
 	/*
 	 * not doing this: it also clears the 'PATH'
@@ -521,7 +527,7 @@ void setup_for_childproc(int fd, char close_fd_0, char *term)
 	 */
 
 	/* set terminal */
-	sprintf(dummy, "TERM=%s", term);
+	snprintf(dummy, term_len, "TERM=%s", term);
 	if (putenv(dummy) == -1)
 		fprintf(stderr, "setup_for_childproc: Could not set TERM environment-variable (%s): %s (%d)\n", dummy, strerror(errno), errno);
 
@@ -669,6 +675,9 @@ char * term_t_to_string(term_t term)
 
 		case TERM_XTERM:
 			return "xterm";
+
+		case TERM_IGNORE:
+			return "dumb";
 	}
 
 	return "dumb";
@@ -683,20 +692,62 @@ void get_now_ts(char *format_str, char *dest, int dest_size)
 	(void)strftime(dest, dest_size, format_str, ptm);
 }
 
+void double_ts_to_str(double ts, char *format_str, char *dest, int dest_size)
+{
+	time_t now = (time_t)ts;
+	struct tm *ptm = localtime(&now);
+	if (!ptm) error_exit("double_ts_to_str: localtime() failed\n");
+
+	(void)strftime(dest, dest_size, format_str, ptm);
+}
+
 void duplicate_re_array(re *pre_in, int n_rein, re **pre_out, int *n_reout)
 {
 	int loop, old_n = *n_reout;
 
 	*n_reout += n_rein;
-	*pre_out = (re *)myrealloc(*pre_out, (*n_reout) * sizeof(re), "duplicate re array");
-
-	for(loop=0; loop<n_rein; loop++)
+	if (*n_reout)
 	{
-		memcpy(&(*pre_out)[loop + old_n], &pre_in[loop], sizeof(re));
-		memset(&(*pre_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
-		(*pre_out)[loop + old_n].regex_str = mystrdup(pre_in[loop].regex_str, "regex str");
-		if (regcomp(&(*pre_out)[loop + old_n].regex, (*pre_out)[loop + old_n].regex_str, REG_EXTENDED))
-			error_exit("duplicate_re_array: regex '%s' suddenly failed to compile\n", (*pre_out)[loop + old_n].regex_str);
+		*pre_out = (re *)myrealloc(*pre_out, (*n_reout) * sizeof(re), "duplicate re array");
+
+		for(loop=0; loop<n_rein; loop++)
+		{
+			memcpy(&(*pre_out)[loop + old_n], &pre_in[loop], sizeof(re));
+
+			memset(&(*pre_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
+			(*pre_out)[loop + old_n].regex_str = mystrdup(pre_in[loop].regex_str, "regex str");
+			compile_re(&(*pre_out)[loop + old_n].regex, (*pre_out)[loop + old_n].regex_str);
+
+			if (pre_in[loop].cmd)
+				(*pre_out)[loop + old_n].cmd = mystrdup(pre_in[loop].cmd, "command to invoke when re matches");
+		}
+	}
+	else
+	{
+		*pre_out = NULL;
+	}
+}
+
+void duplicate_es_array(strip_t *pes_in, int n_esin, strip_t **pes_out, int *n_esout)
+{
+	int loop, old_n = *n_esout;
+
+	*n_esout += n_esin;
+	*pes_out = (strip_t *)myrealloc(*pes_out, (*n_esout) * sizeof(strip_t), "duplicate es array");
+
+	for(loop=0; loop<n_esin; loop++)
+	{
+		memcpy(&(*pes_out)[loop + old_n], &pes_in[loop], sizeof(strip_t));
+
+		LOG("dup type: %d\n", (*pes_out)[loop + old_n].type);
+
+		memset(&(*pes_out)[loop + old_n].regex, 0x00, sizeof(regex_t));
+		(*pes_out)[loop + old_n].regex_str = mystrdup(pes_in[loop].regex_str, "regex str");
+
+		compile_re(&(*pes_out)[loop + old_n].regex, (*pes_out)[loop + old_n].regex_str);
+
+		if (pes_in[loop].del)
+			(*pes_out)[loop + old_n].del = mystrdup(pes_in[loop].del, "delimiter");
 	}
 }
 
@@ -713,6 +764,19 @@ int find_filterscheme(char *name)
 	return -1;
 }
 
+int find_editscheme(char *name)
+{
+	int loop;
+
+	for(loop=0; loop<n_fs; loop++)
+	{
+		if (strcmp(pes[loop].es_name, name) == 0)
+			return loop;
+	}
+
+	return -1;
+}
+
 void compile_re(regex_t *whereto, char *what)
 {
 	/* compile & store regular expression */
@@ -738,3 +802,199 @@ void grow_mem_if_needed(char **what, int *cur_len, int requested_len)
 	if (changed)
 		*what = myrealloc(*what, *cur_len, "grow_mem_if_needed");
 }
+
+int READ(int fd, char *where_to, int max_len, char *for_whom)
+{
+	for(;;)
+	{
+		int rc = read(fd, where_to, max_len);
+		if (rc >= 0)
+			return rc;
+
+		if (errno != EINTR && errno != EAGAIN)
+			error_exit("do_convert: error writing to script '%s'\n", for_whom);
+	}
+}
+
+int get_value_arg(char *par, char *string, valcheck_t check)
+{
+	long int result;
+	int len, loop;
+	int multiplier = 1;
+
+	if (!string)
+		error_exit("%s needs a parameter\n", par);
+
+	len = strlen(string);
+
+	for(loop=0; loop<len; loop++)
+	{
+		if (!isdigit(string[loop]))
+		{
+			char dummy = toupper(string[loop]);
+
+			if (toupper(string[loop + 1]) == 'B' && (dummy == 'K' || dummy == 'M' || dummy == 'G'))
+			{
+				if (dummy == 'K')
+					multiplier = 1024;
+				else if (dummy == 'M')
+					multiplier = 1024 * 1024;
+				else if (dummy == 'G')
+					multiplier = 1024 * 1024 * 1024;
+
+				break;
+			}
+			else
+				error_exit("%s needs a value as parameter\n", par);
+		}
+	}
+
+	result = strtol(string, NULL, 10);
+
+	if (result > INT_MAX || result == LONG_MIN || result == LONG_MAX)
+		error_exit("value %s for parameter %s is too large\n", string, par);
+
+	result *= multiplier;
+
+	if (check == VAL_ZERO_POSITIVE)
+	{
+		if (result < 0)
+			error_exit("value for %s must be >= 0\n", par);
+	}
+	else if (check == VAL_POSITIVE_NOT_1)
+	{
+		if (result < 0 || result == 1)
+			error_exit("value for %s must be >= 0 and not 1\n", par);
+	}
+	else if (check == VAL_POSITIVE)
+	{
+		if (result <= 0)
+			error_exit("value for %s must be > 0\n", par);
+	}
+	else
+		error_exit("get_value_arg: internal error (%d)\n", check);
+
+	return (int)result;
+}
+
+void add_to_iat(int_array_t *piat, int element)
+{
+	if (piat -> n == piat -> size)
+	{
+		piat -> size = (piat -> size?piat -> size:8) * 2;
+		piat -> elements = (int *)myrealloc(piat -> elements, piat -> size * sizeof(int), "int_array_t");
+	}
+
+	(piat -> elements)[piat -> n] = element;
+	(piat -> n)++;
+}
+
+int get_iat_size(int_array_t *piat)
+{
+	return piat -> n;
+}
+
+void init_iat(int_array_t *piat)
+{
+	piat -> elements = NULL;
+	piat -> size = piat -> n = 0;
+}
+
+void free_iat(int_array_t *piat)
+{
+	myfree(piat -> elements);
+
+	init_iat(piat);
+}
+
+int get_iat_element(int_array_t *piat, int index)
+{
+#ifdef _DEBUG
+	if (index >= piat -> n)
+		error_exit("get_iat_element: index out of bounds (current: %d, # elements: %d)\n", index, piat -> n);
+#endif
+
+	return (piat -> elements)[index];
+}
+
+char *gethome(char *user)
+{
+	char *path = NULL;
+
+	for(;;)
+	{
+		struct passwd *p = getpwent();
+
+		if (!p)
+			break;
+
+		if (user != NULL)
+		{
+			if (strcmp(p -> pw_name, user) == 0)
+			{
+				path = mystrdup(p -> pw_dir, "gethome");
+				break;
+			}
+		}
+		else if (p -> pw_uid == getuid())
+		{
+			path = mystrdup(p -> pw_dir, "gethome");
+			break;
+		}
+	}
+
+	endpwent();
+
+	return path;
+}
+
+char *myrealpath(char *in)
+{
+	char *home, *pin;
+	int home_len;
+	char *pout;
+
+	if (in[0] != '~')
+		return mystrdup(in, "myrealpath");
+
+	if (in[1] != '/')
+	{
+		char *user;
+		char *slash = strchr(in, '/');
+		int len;
+
+		if (slash)
+		{
+			len = (int)((slash - in) - 1);
+			pin = slash + 1;
+		}
+		else
+		{
+			len = strlen(in) - 1;
+			pin = in + len + 1;
+		}
+
+		user = (char *)mymalloc(len + 1, "copy of username");
+		memcpy(user, &in[1], len);
+		user[len] = 0x00;
+
+		home = gethome(user);
+
+		myfree(user);
+	}
+	else
+	{
+		home = gethome(NULL);
+		pin = in + 1;
+	}
+
+	if (!home)
+		error_exit("Cannot expand path %s: home directory not found!\n", in);
+
+	home_len = strlen(home);
+	pout = (char *)mymalloc(home_len + strlen(in) + 1, "realpath");
+
+	sprintf(pout, "%s/%s", home, pin);
+
+	return pout;
+}
